/*
 * PodcastPlayer.mainForm
 * author: Makoto Yoshida
 * version: 2008/6/8
 * Microsoft Windows 98 or later
 * Microsoft .NET Framework 2.0 or later
 * Media Control Interface, MCIWnd
 */

// statusPosition deviates in an unpredictable manner depending both on setSpeed and the system.
// Windows Media Player, msdxm.ocx, version 6.4, behavior inconsistent among systems.
// Windows Media Player, wmp.dll, version 9, change speed feature not available in win2k or earlier.

using System;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

namespace PodcastPlayer
{
    public partial class mainForm : Form
    {
        private const int WM_USER = 0x0400;
        private const int MCIWNDF_NOMENU = 0x0008;
        private const int MCIWNDM_GETACTIVETIMER = WM_USER + 132;
        private const int MCIWNDM_SENDSTRING = WM_USER + 101;
        private const int MCIWNDM_RETURNSTRING = WM_USER + 138;
        private const int MCIWNDM_OPEN = WM_USER + 153;

        [DllImportAttribute("user32.dll")]private static extern IntPtr SendMessage(
            IntPtr hWnd, int Msg, IntPtr wParam, IntPtr lParam);
        [DllImportAttribute("user32.dll")]private static extern IntPtr SendMessage(
            IntPtr hWnd, int Msg, IntPtr wParam, StringBuilder lParam);

        [DllImportAttribute("msvfw32.dll")]private static extern IntPtr MCIWndCreate(
            IntPtr hwndParent, IntPtr hInstance, uint dwStyle, StringBuilder szFile); // vfw.h, vfw32.lib

        private IntPtr hWnd;
        private decimal statusLength = 0M;
        private bool pauseMouse = false;
        private bool loopLooping = false;
        private decimal loopStart;
        private decimal loopEnd;
        private decimal speedPosition = 0M;
        private PodcastPlayer.settingsForm theSettingsForm = new PodcastPlayer.settingsForm();

        public mainForm()
        {
            this.InitializeComponent();
            this.frameTableLayoutPanel.Margin =
                new System.Windows.Forms.Padding(0, 0, 0, this.theStatusStrip.Height);
            this.jumpDateTimePicker.Value = this.jumpDateTimePicker.MinDate;

            hWnd = MCIWndCreate(
                mciPanel.Handle, IntPtr.Zero, MCIWNDF_NOMENU, new StringBuilder());
            this.theTimer.Interval =
                (int)SendMessage(hWnd, MCIWNDM_GETACTIVETIMER, IntPtr.Zero, IntPtr.Zero);
            mciSend("set time format milliseconds");
        }

        private void mciSend(string Command)
        {
            if (SendMessage(hWnd, MCIWNDM_SENDSTRING,
                IntPtr.Zero, new StringBuilder(Command)) != IntPtr.Zero) return;
        }

        private string mciSendReturn(string Command)
        {
            StringBuilder sb = new StringBuilder();
            if (SendMessage(hWnd, MCIWNDM_SENDSTRING,
                IntPtr.Zero, new StringBuilder(Command)) != IntPtr.Zero) return null;
            if (SendMessage(hWnd, MCIWNDM_RETURNSTRING,
                new IntPtr(1000), sb) != IntPtr.Zero) return null;
            return sb.ToString();
        }

        private void openFile(string FileName)
        {
            // FileName may also be a URL.
            // A trick is required if the mp3 filename extention is not automatically recognized.
            if (SendMessage(hWnd, MCIWNDM_OPEN, IntPtr.Zero,
                new StringBuilder(FileName.EndsWith(".mp3", StringComparison.OrdinalIgnoreCase)
                    ? "mpegvideo!" + FileName : FileName)) != IntPtr.Zero) return;
            statusLength = decimal.Parse(mciSendReturn("status length").Trim()) / 1000M; // milliseconds

            this.Text = new System.IO.FileInfo(FileName).Name;
            this.pauseCheckBox.Enabled = true;
            this.pardonButton.Enabled = true;
            this.loopLabel.Enabled = true;
            this.speedCheckBox.Enabled = true;
            this.markButton.Enabled = true;
            this.jumpDateTimePicker.Enabled = true;
            this.jumpDateTimePicker.MinDate = DateTimePicker.MinimumDateTime;
            this.jumpDateTimePicker.MaxDate =
                DateTimePicker.MinimumDateTime.AddSeconds((double)statusLength);
            this.jumpDateTimePicker.Value = this.jumpDateTimePicker.MinDate;
            this.jumpButton.Enabled = true;
            this.theTimer.Enabled = true;

            mciSend("play");
        }

        private decimal statusPosition()
        {
            return decimal.Parse(mciSendReturn("status position").Trim()) / 1000M; // milliseconds
        }

        private void seekTo(decimal Seconds)
        {
            if (Seconds < 0M) Seconds = 0M;
            if (statusLength < Seconds) Seconds = statusLength;
            mciSend("seek to " + ((long)(Seconds * 1000M)).ToString().Trim()); // milliseconds
            mciSend("play");
        }

        private void setSpeed(decimal Speed)
        {
            if ((Speed < 0.003M) || (2.268M <= Speed)) return;
            mciSend("set speed " + ((long)(Speed * 1000M)).ToString().Trim());
        }

        private decimal dateTime2Seconds(DateTime Hms)
        {
            return (decimal)((new TimeSpan(Hms.Hour, Hms.Minute, Hms.Second)).TotalSeconds);
        }

        private DateTime seconds2DateTime(decimal Seconds)
        {
            // Ticks in 100 nanoseconds.
            // Has to be greater than MinDate.
            return new DateTime(
                (long)(Seconds * 10000000M) + this.jumpDateTimePicker.MinDate.Ticks);
        }

        private void mainForm_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, true))
            {
                e.Effect = DragDropEffects.Link;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void mainForm_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, true))
            {
                // Opens the first file.
                openFile(((string[])e.Data.GetData(DataFormats.FileDrop, true))[0]);
            }
        }

        private void mainForm_Activated(object sender, EventArgs e)
        {
            this.Opacity = 1D;
        }

        private void mainForm_Deactivate(object sender, EventArgs e)
        {
            if (this.TopMost) this.Opacity = 0.8D;
        }

        private void mainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            mciSend("stop");
            mciSend("close");
            PodcastPlayer.Properties.Settings.Default.Save();
        }

        private void pauseCheckBox_MouseEnter(object sender, EventArgs e)
        {
            if (!this.pauseCheckBox.Checked) mciSend("pause");
            pauseMouse = true;
        }

        private void pauseCheckBox_MouseLeave(object sender, EventArgs e)
        {
            if (!this.pauseCheckBox.Checked) mciSend("play");
            pauseMouse = false;
        }

        private void settingsButton_Click(object sender, EventArgs e)
        {
            if (!theSettingsForm.Visible) theSettingsForm.Show(this);
        }

        private void aboutButton_Click(object sender, EventArgs e)
        {
            MessageBox.Show(aboutProduct() + " version " + aboutVersion() + "\n" +
                aboutCopyright(), aboutTitle());
        }

        private string aboutTitle()
        {
            object[] o = Assembly.GetExecutingAssembly().GetCustomAttributes(
                typeof(AssemblyTitleAttribute), false);
            if (o.Length != 0)
            {
                return ((AssemblyTitleAttribute)o[0]).Title;
            }
            else
            {
                return string.Empty;
            }
        }

        private string aboutProduct()
        {
            object[] o = Assembly.GetExecutingAssembly().GetCustomAttributes(
                typeof(AssemblyProductAttribute), false);
            if (o.Length != 0)
            {
                return ((AssemblyProductAttribute)o[0]).Product;
            }
            else
            {
                return string.Empty;
            }
        }

        private string aboutVersion()
        {
            return Assembly.GetExecutingAssembly().GetName().Version.ToString();
        }

        private string aboutCopyright()
        {
            object[] o = Assembly.GetExecutingAssembly().GetCustomAttributes(
                typeof(AssemblyCopyrightAttribute), false);
            if (o.Length != 0)
            {
                return ((AssemblyCopyrightAttribute)o[0]).Copyright;
            }
            else
            {
                return string.Empty;
            }
        }

        private void pardonButton_Click(object sender, EventArgs e)
        {
            seekTo(statusPosition() - this.repeatNumericUpDown.Value);
        }

        private void loopLabel_MouseEnter(object sender, EventArgs e)
        {
            loopLooping = true;
            loopEnd = statusPosition();
            loopStart = loopEnd - this.repeatNumericUpDown.Value;
            this.loopLabel.BorderStyle = BorderStyle.FixedSingle;
        }

        private void loopLabel_MouseLeave(object sender, EventArgs e)
        {
            loopLooping = false;
            this.loopLabel.BorderStyle = BorderStyle.Fixed3D;
        }

        private void repeatNumericUpDown_Validated(object sender, EventArgs e)
        {
            PodcastPlayer.Properties.Settings.Default.Save();
        }

        private void speedNumericUpDown_ValueChanged(object sender, EventArgs e)
        {
            if (this.speedCheckBox.Checked) setSpeed(this.speedNumericUpDown.Value);
        }

        private void speedNumericUpDown_Validated(object sender, EventArgs e)
        {
            PodcastPlayer.Properties.Settings.Default.Save();
        }

        private void speedCheckBox_MouseEnter(object sender, EventArgs e)
        {
            if (!this.speedCheckBox.Checked)
            {
                speedPosition = statusPosition();
                setSpeed(this.speedNumericUpDown.Value);
                if (this.pauseCheckBox.Checked) mciSend("pause");
            }
        }

        private void speedCheckBox_MouseLeave(object sender, EventArgs e)
        {
            if (!this.speedCheckBox.Checked)
            {
                seekTo(speedPosition);
                setSpeed(1M);
                if (this.pauseCheckBox.Checked) mciSend("pause");
            }
        }

        private void speedCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (!this.speedCheckBox.Checked)
            {
                speedPosition = statusPosition();
                if (this.pauseCheckBox.Checked) mciSend("pause");
            }
        }

        private void markButton_Click(object sender, EventArgs e)
        {
            this.jumpDateTimePicker.Value = seconds2DateTime(statusPosition());
        }

        private void jumpButton_Click(object sender, EventArgs e)
        {
            seekTo(dateTime2Seconds(this.jumpDateTimePicker.Value));
        }

        private void theTimer_Tick(object sender, EventArgs e)
        {
            if (!pauseMouse)
            {
                if (string.Compare(mciSendReturn("status mode"), "playing") == 0)
                {
                    this.pauseCheckBox.Checked = false;
                }
                else
                {
                    this.pauseCheckBox.Checked = true;
                }
            }

            if (loopLooping && (loopEnd < statusPosition())) seekTo(loopStart);

            this.theToolStripStatusLabel.Text =
                seconds2DateTime(statusPosition()).ToString("HH:mm:ss")
                + " / " + seconds2DateTime(statusLength).ToString("HH:mm:ss");
        }
    }
}